Type TVector
	Field x:Float
	Field y:Float
	Field index:Int = 0
	Method Subtract(other:TVector)
		x :- other.x
		y :- other.y
	End Method
	Method Add(other:TVector)
		x :+ other.x
		y :+ other.y
	End Method
	Method Normalize()
		Local magnitude:Float = Sqr(x*x+y*y)
		x = x / magnitude
		y = y / magnitude
	End Method
End Type

Type TPlane
	Field n:TVector
	Field d:Float
	Field v0:TVector
	Field v1:TVector
	Field e:Int = True
	Method FlipNormals()
		n.x = -n.x
		n.y = -n.y
		d = -d
	EndMethod
End Type

Function MakeVector:TVector(xx:Float, yy:Float)
	Local v:TVector= New TVector
	v.x = xx
	v.y = yy
	Return v
End Function

Function NormalizeVector:TVector(v:TVector)
	Local magnitude:Float = VectorLength(v)
	v.x = v.x / magnitude
	v.y = v.y / magnitude
	Return v
End Function

Function DotVector:Float(v0:TVector, v1:TVector)
	Return (v0.x * v1.x) + (v0.y * v1.y)
End Function

Function PlaneDistance:Float(p:TPlane, v:TVector)
	Return DotVector(v, p.n) - p.d
End Function

Function AddVector:TVector(v0:TVector, v1:TVector)
	Local r:TVector = New TVector
	r.x = v0.x + v1.x
	r.y = v0.y + v1.y
	Return r
End Function

Function SubtractVector:TVector(v0:TVector, v1:TVector)
	Local r:TVector = New TVector
	r.x = v0.x - v1.x
	r.y = v0.y - v1.y
	Return r
End Function

Function MultiplyVector:TVector(v0:TVector, v1:TVector)
	Local r:TVector = New TVector
	r.x = v0.x * v1.x
	r.y = v0.y * v1.y
	Return r
End Function

Function VectorLength:Float(v0:TVector)
	Return Sqr(v0.x*v0.x + v0.y*v0.y)
End Function

Function MakePlane:TPlane(v0:TVector, v1:TVector)
	Local dis:TVector = NormalizeVector(SubtractVector(v1, v0))
	Local normal:TVector = MakeVector(dis.y, -dis.x)
	Local p:TPlane = New TPlane
	p.n = normal
	p.d = DotVector(normal, v0)
	p.v0 = v0
	p.v1 = v1
	Return p
End Function